// Load Mongoose OS API
load('api_sys.js');
load('api_config.js');
load('api_events.js');
load('api_gpio.js');
load('api_log.js');
load('api_timer.js');
load('api_aws.js');
load('api_mqtt.js');
load('api_shadow.js');


// init hw to see output and generate actions
let led_pin = 14;
let button_pin = 0;
GPIO.set_mode(led_pin, GPIO.MODE_OUTPUT);
GPIO.set_mode(button_pin, GPIO.MODE_INPUT);

// Device state - output on/off status, on = true;
let state = {
    out1: false,
    out2: false
    // ...
};

let teletopic = Cfg.get('device.id') + '/telemetry';
let buttopic = Cfg.get('device.id') + '/buttons';


// send telemetry
function sendTelemetry() {
  if(AWS.isConnected()){
    let msg = JSON.stringify({free: Sys.free_ram(), total: Sys.total_ram()}); 
    Log.print(Log.INFO, 'Send telemetry:' + ((MQTT.pub(teletopic, msg)) ? 'OK' : 'FAIL') + ' msg:' +  msg);
  }
};

// send user action
function sendAction() {
  if(AWS.isConnected()){
    let msg = JSON.stringify({button1: true}); 
    Log.print(Log.INFO, 'Send action:' + ((MQTT.pub(buttopic, msg)) ? 'OK' : 'FAIL') + ' msg:' +  msg);
  }
};

// operate output 1
function handleOutput1(onoff, localcontrol) {
  Log.print(Log.DEBUG, 'output1 is ' + (state.out1 ? 'ON':'OFF') + ', requested to be ' + (onoff ? 'ON':'OFF'));
  GPIO.write(led_pin, onoff ? 1 : 0);
  state.out1 = onoff;
  Log.print(Log.INFO, 'output1 is now: ' + (state.out1 ? 'ON':'OFF'));
  if(AWS.isConnected() && localcontrol){
//  AWS.Shadow.update_ext(0, {desired: {out1: state.out1}, reported: state});
//  AWS.Shadow.update_ext(0, {desired: {out1: state.out1}});
  }
};

// periodically send telemetry
Timer.set(10000 /* milliseconds */, Timer.REPEAT, sendTelemetry, null);

// send user action and toggle output on button press
GPIO.set_button_handler(button_pin, GPIO.PULL_NONE, GPIO.INT_EDGE_POS, 100, function() {
  sendAction();
//handleOutput1(!state.out1, true); // toggle state, send update now
}, null);


// Set up Shadow handler to synchronise device state (outputs) with the shadow state
Shadow.addHandler(function(event, obj) {
    if (event === 'CONNECTED') {
      // Connected to shadow - report our current state.
      Shadow.update(0, state);
      Log.print(Log.INFO, 'Connected to AWS');
    } else if (event === 'UPDATE_DELTA') {
      Log.print(Log.DEBUG, 'Got DELTA');
      // Got delta. Iterate over the delta keys, handle those we know about.
      for (let key in obj) {
        if (key === 'out1') {
          handleOutput1(obj.out1, false); // don't send update now
        } else if (key === 'out2') {
          state.out2 = obj.out2;
          Log.print(Log.INFO, 'output2 is now: ' + ((state.out2)? 'ON':'OFF'));
        } /* ...*/ else {
            Log.print(Log.WARN, 'Unknown key: ' + key);
        }
      }
      // Once we've done synchronising with the shadow, report our state.
      Shadow.update(0, state);
    }
  });
  



// generic cloud events
Event.addHandler(Event.CLOUD_CONNECTED, function (ev, evdata, ud) {
    Log.print(Log.INFO, 'Connected to cloud');
}, null);
Event.addHandler(Event.CLOUD_DISCONNECTED, function (ev, evdata, ud) {
    Log.print(Log.INFO, 'Disconnected from cloud');
}, null);


Log.print(Log.WARN, "### init script started ###");

